###' constant transmission matrix
###'
###' construct a transmission matrix generator with no parameters
###' @param M a matrix
###' @param R vector of region names
###' @return a function that returns M for all time points
###' @export
###' @examples
###' M = constantTrans(matrix(1:25,ncol=5),R=letters[1:5])
###' M(0)
###' M(1)

constantTrans <- function(M, R=row.names(M)){
  if(missing(R) & missing(M)){
    stop("No matrix or region names given")
  }
  if(missing(M)){
    M = matrix(1,ncol=length(R),nrow=length(R))
  }
  if(!missing(M) & !missing(R)){
    row.names(M) <- R
  }else{
    if(is.null(row.names(M))){
      stop("Matrix argument has no row names for regions")
    }
  }
  if(is.null(R)){
    stop("No region names specified or taken from M")
  }
  if(nrow(M)!=ncol(M)){
    stop("Tranmissivity matrix not square")
  }
  row.names(M)=R
  colnames(M)=R
  ct = function(t,gamma){
    return(M)
  }
  attr(ct,"ngamma")=0
  attr(ct,"tconst")=TRUE
  attr(ct,"size") = nrow(M)
  attr(ct,"regions")=row.names(M)
  class(ct) <- c("plimtrans","fixedTrans","function")
  return(ct)
}
###' print for fixed transmission generators
###'
###' print method for these things
###'
###' @param x a transmission matrix generator
###' @param ... other args, unused for now
###' @export
print.fixedTrans <- function(x,...){
  cat("Fixed Transmission Model for",attr(x,"size"),"regions\n")
#  cat("To evaluate, try ",name,"(0) (returns a matrix)\n")
}

###' distance decay transmission matrix
###'
###' construct a transmission matrix generator for distance decay
###' @param coords either a 2-column coordinate matrix or a full NxN distance matrix
###' @param R vector of region names - can be taken from row names of coords
###' @return a function that returns the distance decay matrix
###' @examples
###' M = distDecayTrans(cbind(1:5,1:5),R=letters[1:5])
###' M(0,c(0,0))
###' M(0,c(1,1))
###' @export
distDecayTrans <- function(coords,R=row.names(coords)){
  if(ncol(coords)==2){
    dmat = as.matrix(dist(coords))
  }else{
    dmat = coords
  }
  ct = function(t,gamma){
    return(1+exp(gamma[1])*exp(-(dmat)/exp(gamma[2])))
  }
  attr(ct,"ngamma") = 2
  if(is.null(R)){
    stop("No region names given or obtained from coords")
  }
  attr(ct,"regions")=R
  attr(ct,"tconst") = TRUE
  attr(ct,"size") = nrow(dmat)
  class(ct) <- c("plimtrans","distDecayTrans","function")
  
  return(ct)
}

###' transmissivity as an increasing linear function
###'
###' C is exp(gamma1) + wij * exp(gamma2)
###'
###' @param mat square matrix of values
###' @param R row names - can be taken from matrix
###' @return a function that returns C
###' @examples
###' M = incLinearTrans(matrix(1:16,4,4),R=letters[1:4])
###' M(0,c(1,1))
###' M(0,c(1,2))
###' @export
incLinearTrans <- function(mat,R=row.names(mat)){

  if(is.null(R)){
    stop("No region names given or obtained from matrix")
  }

  .checkMat(mat,R)

  force(mat)
  ct = function(t,gamma){
    return(exp(gamma[1])+exp(gamma[2])*mat)
  }
  attr(ct,"ngamma") = 2
  attr(ct,"regions")= R
  attr(ct,"tconst") = TRUE
  attr(ct,"size") = nrow(mat)
  class(ct) <- c("plimtrans","incLinearTrans","function")
  return(ct)
}

###' basic sanity checks
###'
###' 2d square matrix with correct region codes
###'
###' @param mat square matrix
###' @param R region codes

.checkMat <- function(mat,R){
  dm=dim(mat)
  if(length(dm)!=2){
    stop("Matrix not 2-d")
  }
  if(dm[1]!=dm[2]){
    stop("Matrix not square")
  }
  if(length(R)!=dm[1]){
    stop("Wrong number of region codes (",length(R),") for a ",dm[1],"x",dm[2]," matrix.")
  }
}

###' general piece-wise linear generator
###'
###' piecewise linear given breakpoints
###'
###' @param mat square matrix of values
###' @param brks vector of breakpoints
###' @param R row names - can be taken from matrix
###'
###' @export
###'
pieceLinearTrans <- function(mat,brks,R=row.names(mat)){
  if(length(brks)<2){
    stop("Must be at least two break points")
  }
  if(is.null(R)){
    stop("No region names given or obtained from matrix")
  }
  
  .checkMat(mat,R)

  force(mat)
  force(brks)
  ngamma = length(brks)
  bi = as.numeric(cut(mat,c(-Inf,brks[-c(1,length(brks))],Inf)))
  f = (mat-brks[bi])/(brks[bi+1]-brks[bi])
  ct = function(t,gamma){
    stopifnot(length(gamma)==ngamma)
    v = gamma[bi] + f * (gamma[bi+1]-gamma[bi])
    return(v)
  }
  attr(ct,"ngamma")=ngamma
  attr(ct,"regions")=R
  attr(ct,"tconst")=TRUE
  attr(ct,"size")=nrow(mat)
  class(ct) <- c("plimtrans","pieceLinearTrans","function")
  return(ct)
  
}

###' print a piece-wise linear generator
###'
###' print method for piecewise linear generator
###'
###' @param x a piece-wise linear generator
###' @param ... other args
###' @export
print.pieceLinearTrans <- function(x,...){
  cat("Piece-wise linear model for",attr(x,"size"),"regions\n")
  cat("Breakpoints at ",with(environment(x),brks),"\n")
}

###' print a linear increasing generator
###'
###' print method for linear increasing generator function
###'
###' @param x a linear increasing generator
###' @param ... other args
###' @export
print.incLinearTrans <- function(x,...){
  cat("Increasing Linear Model for",attr(x,"size"),"regions\n")
  cat("   exp(gamma[1]) + exp(gamma[2] * mat\n")
}

###' print a distance-decay generator
###'
###' distance-decay generator print method
###' 
###' @param x a distance-decay generator
###' @param ... other args, unused for now
###' @export
print.distDecayTrans <- function(x,...){
  cat("Distance Decay Model for",attr(x,"size"),"regions\n")
  cat("   1+exp(gamma[1])*exp(-(dmat)/exp(gamma[2]))\n")
}

###' general transmission matrix generator
###'
###' construct a possibly time-varying transmission matrix generator
###' @param g function of time and gamma
###' @param ngamma number of gamma parameters
###' @param ... other arguments passed to g
###' @return matrix generator function of time and gamma
###' @export
timeVaryingTrans <- function(g,ngamma,...){

  ct = function(t,gamma){
    return(g(t,gamma,...))
  }
  attr(ct,"ngamma")=ngamma
  attr(ct,"tconst")=FALSE
  class(ct) <- c("plimtrans","function")
  return(ct)
}

###' print a transmission matrix generator
###'
###' @param x a transmission matrix generator
###' @param ... other args, unused for now
###' @export
print.plimtrans <- function(x,...){
  cat("Transmission Matrix Generator\n")
  NextMethod("print",x,...)
}

###' plot the distance curve of a transmission matrix generator
###'
###' @param x transmission matrix generator
###' @param y parameter vector
###' @param ... time for plot
###' @export

plot.distDecayTrans <- function(x,y,...){
  dm = with(environment(x),dmat)
  xc = dm[order(dm)]
  yc = x(t,y)[order(dm)]
  plot(xc,yc,type="l",xlab="Distance",ylab="Log Relative Rate",
       ylim=c(min(min(yc,0.95)),max(yc)),lwd=3,col="#8DA0CB",log="y")
  abline(h=1,lty=2)
}

##' plot method for transmission objects
##'
##' @param x
##' @param y
##' @param ...

##' @return ...
plot.plimtrans <- function(x,y,...){
  NextMethod("plot",x)
}

##' plot method for fixed transmission objects
##'
##' @param x
##' @param y
##' @param ...

##' @return ...
plot.fixedTrans <- function(x,y,...){
  cat("no way\n")
}

###' time-fixed covariate matrix generator
###'
###' construct a time-fixed covariate matrix generator
###' @param M matrix or data frame of covariates
###' @param R vector of region names
###' @return a function that returns M for all time values
###' @export
constantCovariates <- function(M,R=rownames(M)){

  if(missing(M)){
    if(missing(R)){
      stop("No region names given")
    }
    M=matrix(9999,1,1) # need to pass something here
    ct = function(t){
      return(M)
    }
    attr(ct,"size")=0
    attr(ct,"covarNames")=""
    attr(ct,"regions")=R
    attr(ct,"tconst")=TRUE
    class(ct) <- c("plimcovariates","function")
    return(ct)
  }

  if(!missing(R)){
    if(!all(rownames(M)==R)){
      warning("Row names replaced by R or in different order to those in input matrix/data frame")
    }
  }
  
  M = as.matrix(M,rownames.force=TRUE)
  row.names(M)=R
  n = ncol(M)
  if(is.null(colnames(M))){
    colnames(M)=paste("Z",1:n,sep="")
  }
  ct = function(t){
    return(M)
  }
  attr(ct,"size")=ncol(M)
  attr(ct,"covarNames")=colnames(M)
  if(is.null(rownames(M))){
    stop("No region names either as row names or as R parameter")
  }
  attr(ct,"regions")=rownames(M)
  attr(ct,"tconst")=TRUE
  class(ct) <- c("plimcovariates","function")
  return(ct)
}

###' print the covariate matrix generator
###'
###' @param x a covariate matrix generator
###' @param ... other args, unused for now
###' @export
print.plimcovariates <- function(x,...){
  cat("Covariate generator function\n")
  if(attr(x,"size")==0){
    cat("Null model, no explanatory variables\n")
    return()
  }
  cat(length(attr(x,"regions")),"regions\n")
  cat(attr(x,"size"),"explanatory variables ")
  cat("(",paste(attr(x,"covarNames"),collapse=","),")\n",sep="")

  if(isTRUE(attr(x,"tconst"))){
    cat("Time-invariant\n")
  }else{
    cat("Time-varying\n")
  }
  
}
