###' fit the model using the fast C code
###'
###' fit a partial-likelihood infectious model
###'
###' conditional on the times, we compute the likelihood that the given cases occurred in the
###' given regions by computing p_k for each case. p_k depends on infectivity A of all other
###' regions, susceptibility B of the regions in question, transmissivity C between the region
###' and all other regions, population of the regions, and the number of infectious cases in the
###' regions. For more details, there's probably a paper you should read.
###' 
###' @param paramList list of alpha, beta, gamma parameters
###' @param cases region-time case data
###' @param window infectivity timespan
###' @param pop population vector
###' @param Z covariate generator
###' @param W interaction term generator
###' @param control optim control parameter - fnscale is set to -1
###' @param hessian flag to return hessian or not
###' @param method optim method parameter
###' @param code character code - whether to use C or R 
###' @param ... extra parameters passed to likelihood function
###' @export
###' @return a fitted plim model object
###' @examples
###' m = makeData()
###' p0 = list(alpha=c(0,0),beta=c(0,0))
###' f = plim(p0,m$cases,m$window,m$pop,m$Z,m$W)
###' plimtable(f)

plim <-  function(paramList, cases, window, pop, Z, W, control=list(maxit=500),hessian=TRUE,method="Nelder-Mead",code="C",...){

  p0 = buildParams(paramList)
  nameCheck(cases,Z,W)

  if(code=="C"){
    if(attr(Z,'tconst') & attr(W,'tconst')){
      ## can use C code:
      return(transModelCFit(p0,cases,window,pop,Z,W,control=control,hessian=hessian,method=method,...))
    }else{
      stop("Can't deal with time-varying things at the moment")
    }
  }
  ## use R code if asked:
  if(code == "R"){
    if(attr(Z,'tconst') & attr(W,'tconst')){
      stop("fitting with R code not implemented at the moment")
      ## TODO
      ### return(transModelFit(p0,cases,window,pop,Z,W,control=control,hessian=hessian,method=method))  
    }else{
      stop("Can't deal with time-varying things at the moment")
    }
  }
}

###' compute a likelihood for the plim model
###'
###' just returns one likelihood value for the parameters
###'
###' @param paramList list of alpha, beta, gamma values
###' @param cases region-time case data
###' @param window infectivity timespan
###' @param pop population vector
###' @param Z covariate generator
###' @param W transmissivity generator
###' @param mc count of infectious cases per region per day - computed if not given
###' @param code whether to use "C" or "R" code.
###' @export
###' @return the log-likelihood
###' 

plimlik <- function(paramList, cases, window, pop, Z, W, mc, code="C"){
  R = attr(W,"regions")
  if(missing(mc)){
    mc = mCounts(window, cases, R)
  }
  pars = buildParams(paramList)
  likFun = switch(code,
    C = transModelCLik,
    R = transModelLik,
    stop("Invalid code letter")
    )
  return(likFun(pars,cases, pop, Z, W, mc))  
}

###' build parameter vector from list
###'
###' convert list(alpha=,beta=,gamma=) to a named vector
###'
###' @param pl list of alpha, beta, gamma parameters
###' @return named vector of parameters
buildParams <- function(pl){
  nalpha = length(pl$alpha)
  nbeta = length(pl$beta)
  ngamma = length(pl$gamma)
  p = c(pl$alpha,pl$beta,pl$gamma)
  names(p)=c(rep("alpha",nalpha),rep("beta",nbeta),rep("gamma",ngamma))
  return(p)
}

###' check all region names are correct
###'
###' make sure they all agree...
###'
###' @param cases region-time case data
###' @param Z covariate generator
###' @param W interaction term generator
###' @return nothing, raises an error on fail
###' @export

nameCheck <- function(cases, Z, W){
  caseRegs = caseRegions(cases)
  Zregions = attr(Z,"regions")
  Wregions = attr(W,"regions")
  errors = ""
  if(length(Wregions)!=length(Zregions)){
    stop("Regions from W are not the same length as from Z")
  }

  if(!all(Zregions==Wregions)){
    stop("Regions specified in W are not those in Z")
  }
### we get here, it means W and Z all have the same regions
  if(!all(caseRegs %in% Zregions)){
    stop("Some Regions in the cases are not in the region codes from W and Z")
  }
  
}
