###' Simulates from Transmission model
###'
###' simulates data for the given parameters
###'
###' @param params initial parameters
###' @param cases region-time case data
###' @param window infectivity timespan
###' @param R region codes
###' @param pop population vector
###' @param Z covariate generator
###' @param W interaction term generator
###' @param mC counts of cases at times in regions
###' @param seed random number seed
###' @return data frame of time and simulated region
###' @export
###' @examples
###' set.seed(123)
###' m = makeData()
###' mc = mCounts(0.3, m$cases, m$R)
###' plimsim(c(alpha=0,alpha=0,beta=0,beta=0),m$cases,0.3,R=m$R,pop=m$pop,Z=m$Z,m$W,mc)
plimsim <- function(params, cases, window, R, pop, Z, W, mC,seed=NULL){
###
  if(!is.null(seed)){
    set.seed(seed)
  }
  ct = caseTimes(cases)
  crc = caseRegions(cases,character=FALSE)
  cr = match(caseRegions(cases),attr(W,"regions"))
  
  alpha = params[names(params)=="alpha"]
  beta = params[names(params)=="beta"]
  gamma = params[names(params)=="gamma"]
  case <- c()
  for(r in 1:nrow(cases)){
    t = ct[r]
    A = exp(Z(t) %*% alpha)
    B = exp(Z(t) %*% beta)
    C = W(t, gamma)
    
    lambdaDash = outer(as.vector(A),as.vector(B)) * C
    
    ## get number of cases in the history for each region
    mt = mC[r,]
    ## skip if there's no contributing cases in the history:
    if (all(mt==0)){
      case[r] <- as.character(crc[r])
    }
    else{
      p <- c()
      b1 = colSums(mt * lambdaDash)
      bottom = sum(pop * b1)
      top <- sapply(1:length(R),function(k){return(pop[k] * sum(mt * lambdaDash[,k]))})
      p <- top/bottom
      if(any(!is.numeric(p))){
        warning("Something went non-numeric in likelihood calculation.")
        browser()
      }
      if(any(is.na(p))){
        warning("Perhaps try some other parameter values ... ?")
      }
      case[r] <- sample(R,1,prob=p)
    }	    
  }
  df <- data.frame(region=case,time=ct)
  df <- stCases(df,"region","time")
  return(df)
}


