
###' fit the model using the fast C code
###'
###' uses speedy C code to fit the likelihood
###'
###' @param params initial parameters
###' @param cases region-time case data
###' @param window infectivity timespan
###' @param pop population vector
###' @param Z covariate generator
###' @param W interaction term generator
###' @param control optim control parameter - fnscale is set to -1
###' @param hessian flag to return hessian or not
###' @param method optim method parameter
###' @param ... extra parameters passed to likelihood function
###' @export
###' @useDynLib plim
###' @examples
###' m = makeData()
###' p0 = c(0,0,0,0)
###' names(p0)=c("alpha","alpha","beta","beta")
###' f = transModelCFit(p0,m$cases,m$window,m$pop,m$Z,m$W)
###' plimtable(f)
transModelCFit <-  function(params, cases, window,pop, Z, W, control=list(maxit=500),hessian=TRUE,method="Nelder-Mead",...){
  require(plyr)
  R = attr(Z,"regions")
  mC = mCounts(window,cases,R) # laply(caseTimes(cases,numeric=TRUE),function(x){mCount(x,window,cases,R)})
  control$fnscale=-1
  op = optim(params,transModelCLik,
    cases=cases,pop=pop,Z=Z,W=W,mC=mC,
    control=control,hessian=hessian,method=method,...)
  covarNames = c(
    paste("alpha",attr(Z,'covarNames'),sep="."),
    paste("beta",attr(Z,'covarNames'),sep=".")
    )
  if(attr(Z,"size")>0){
    names(op$par)[1:length(covarNames)]=covarNames
    if(hessian){
      dnames = dimnames(op$hessian)[[1]]
      dnames[1:length(covarNames)]=covarNames
      dimnames(op$hessian)=list(dnames,dnames)
    }
  }
    
  return(op)
}

###' Transmission model likelihood
###'
###' compute the likelihood for the given parameters
###'
###' @param params initial parameters
###' @param cases region-time case data
###' @param pop population vector
###' @param Z covariate generator
###' @param W interaction term generator
###' @param mC counts of cases at times in regions
###' @param trace level of logging and debugging
###' @return the log-likelihood given the parameters
###' @export
###' @examples
###' m = makeData(seed=123)
###' mc = mCounts(m$window, m$cases, m$R)
###' transModelCLik(c(alpha=0,alpha=0,beta=0,beta=0),m$cases,pop=m$pop,Z=m$Z,m$W,mc)

transModelCLik <- function(params, cases, pop, Z, W, mC, trace=0){
###

  if(trace>0){
    cat("Params: \n")
    print(params)
  }
  ll = 0
  ct = caseTimes(cases,numeric=TRUE)
  cr = match(caseRegions(cases),attr(W,"regions"))
  alpha = params[names(params)=="alpha"]
  beta = params[names(params)=="beta"]
  gamma = params[names(params)=="gamma"]

  ncovars = attr(Z,'size')
  nregions = length(attr(W,"regions"))
  ncases = length(ct)

  Z=Z(0)
  W = W(0,gamma)
  
  stopifnot(ncovars == length(alpha),ncovars == length(beta))

  
  lambda = matrix(0,nregions,nregions)
  if(trace>0){
    callC = list(
      alpha=as.double(alpha),
      beta=as.double(beta),
      ncovars = as.integer(ncovars),
      
      ct = as.double(ct),
      cr = as.integer(cr),
      ncases = as.integer(ncases),
            
      pop = as.double(pop),
      nregions = as.integer(nregions),
      
      Z = as.double(Z),
      W = as.double(W),
      mC = as.integer(mC),
      lambda = as.double(lambda),
      
      ll = as.double(0)
      )
    badValues = unlist(lapply(callC,function(x){any(!is.finite(x))}))
    if(any(badValues)){
      print("Bad values in args for .C in....")
      print(names(callC)[badValues])
      if(trace > 2){
        browser()
      }
    }

  }

  result = .C("translik",
    as.double(alpha),
    as.double(beta),
    as.integer(ncovars),

    as.double(ct),
    as.integer(cr),
    as.integer(ncases),

    as.double(pop),
    as.integer(nregions),

    as.double(Z),
    as.double(W),
    as.integer(mC),
    as.double(lambda),

    ll = as.double(0)
    )
  if(trace>0){
    cat("ll = ",result$ll,"\n")
  }
  return(result$ll)
}
