###' do likelihood "profiles" in several dimensions...
###'
###' compute the likelihood around a given point
###'
###' @param fit a fitted model from transModelCFit
###' @param fitData the data used in the fit
###' @param dimension how many dimensions to go into
###' @param cstep size of step for covariate parameters
###' @param gstep size of step for gamma parameters
###' @param nsteps number of steps either side of fit
###' @param check whether to test if the fit logl is still the same
###' @param verbose print progress
###' @return a data frame of parameters and the computed likelihood
###' @export
###' @examples
###' m = makeData(seed=56)
###' p0 = c(0,0,0,0)
###' names(p0)=c("alpha","alpha","beta","beta")
###' f = transModelCFit(p0,m$cases,.2,m$pop,m$Z,m$W)
###' plimtable(f)
###' #pf = transProf(f,list(params=p0,cases=m$cases,window=.2,pop=m$pop,Z=m$Z,W=m$W))

transProf <- function(fit,fitData,dimension=1, cstep=0.01, gstep=0.01, nsteps=2, check=FALSE, verbose=FALSE){
  #stopifnot(dimension==1)
  R = attr(fitData$Z,'regions')
  mc =   mCounts(fitData$window, fitData$cases, R)
  ## first point at the fitted location
  p = fit$par
  ## p has names like alpha.covar1, alpha.covar2 etc, so make them plain alpha, beta:
  names(p)=makeParNames(fitData)
  alpha0 = p[names(p)=="alpha"]
  beta0  = p[names(p)=="beta"]
  gamma0 = p[names(p)=="gamma"]

  ##f0=NA
  if(check){
    f0 = transModelCLik(p,fitData$cases, fitData$pop, Z=fitData$Z, W=fitData$W, mC=mc)
    ##  fR = transModelLik(p,fitData$cases, fitData$pop, Z=fitData$Z, W=fitData$W, mC=mc)
    ##  print(fR)
    if(f0!=fit$value){
      warning("value from model fit ",fit$value," not the same when evaluated at fit point ",f0," - something changed?")
    }
  }
  mPF = matrix(nrow=0,ncol=length(p)+1)

  steps=(-nsteps):nsteps
  alphas = sapply(alpha0,function(x){x+steps*cstep})
  betas = sapply(beta0,function(x){x+steps*cstep})
  gammas = sapply(gamma0,function(x){x+steps*gstep})

  if(length(alpha0)==0){
      d = cbind(gammas)
    } else {
      d = cbind(alphas,betas)
      if(length(gamma0)>0){
        d=cbind(d,gammas)
      }
    }

  d = as.list(data.frame(d))
  d$dimension=dimension
  pgrid = do.call(superGrid,d)
  indexes = attr(pgrid,'indices')
  colnames(indexes)=paste("I",names(p),sep="")
  ps = as.matrix(pgrid)

  for(ip in 1:nrow(ps)){
    if(verbose){
      cat(ip,"/",nrow(ps),"\n")
    }
    pp = as.vector(ps[ip,])
    names(pp)=names(p)
    logl = transModelLik(pp,fitData$cases, fitData$pop, Z=fitData$Z, W=fitData$W, mC=mc)
    mPF = rbind(mPF,c(pp,logl))
  }
  
  mPF=data.frame(mPF,indexes)
  
  names(mPF)=c(names(fit$par),"logL",paste("I",names(fit$par),sep=""))
  attr(mPF,"mid")=(2*nsteps+2)/2
  return(mPF)
  
}

###' work out parameter names
###'
###' @param fitData list of parameters fitted
###' @return vector of alpha, beta, gamma
###' @export

makeParNames <- function(fitData){
  nalphabeta = attr(fitData$Z,"size")
  ngamma = attr(fitData$W,"ngamma")
  return(rep(c("alpha","beta","gamma"),c(nalphabeta,nalphabeta,ngamma)))
}

###' return slices, lines, cubes from a full grid
###' @param ... vectors spanning the space (all odd length)
###' @param dimension dimension of the returned features
###' @return matrix of values in n-dimensional parts
###' @export
superGrid <- function(...,dimension){
###
  vs = list(...)
  nd = length(vs)
  lens = unlist(lapply(vs,length))
  if(!all(lens==lens[1])){
    stop("All vectors must be same length until I figure this out")
  }
  if (!(all(lens %% 2 == 1))){
    stop("Even number of values to superGrid")
  }
  seqs = lapply(lens,function(vl){1:vl})
  midpt = 1+(lens[1]-1)/2
  full = do.call(expand.grid,seqs)
  ncents = apply(full, 1, function(r){sum(r==midpt)})
  crit = ncents >= (nd-dimension)
  fullv = do.call(expand.grid,vs)
  values = fullv[crit,]
  indexes = full[crit,]
  attr(values,'indices')=indexes
  return(values)

}
